@@ -1,6 +1,7 @@
 # This file is part of Jaxley, a differentiable neuroscience simulator. Jaxley is
 # licensed under the Apache License Version 2.0, see <https://www.apache.org/licenses/>
 
+from jaxley.__version__ import __version__
 from jaxley.connect import (
     connect,
     connectivity_matrix_connect,
@@ -8,6 +9,16 @@ from jaxley.connect import (
     sparse_connect,
 )
 from jaxley.integrate import integrate
+from jaxley.io.swc import read_swc
 from jaxley.modules import *
 from jaxley.optimize import ParamTransform
 from jaxley.stimulus import datapoint_to_step_currents, step_current
+
+__all__ = [
+    "read_swc",
+    "Module",
+    "Branch",
+    "Cell",
+    "Compartment",
+    "Network",
+]
