@@ -3,6 +3,8 @@
 
 from jaxley.modules.base import Module
 from jaxley.modules.branch import Branch
-from jaxley.modules.cell import Cell, read_swc
+from jaxley.modules.cell import Cell
 from jaxley.modules.compartment import Compartment
 from jaxley.modules.network import Network
+
+__all__ = ["Module", "Branch", "Cell", "Compartment", "Network"]
