@@ -5,7 +5,9 @@ from morph_tool import convert
 
 
 def run_conversion():
-    convert("morph_tmp.swc", "morph.swc", sanitize=True, single_point_soma=True)
+    convert(
+        "morph_tmp.swc", "morph_ca1_n120.swc", sanitize=True, single_point_soma=True
+    )
 
 
 if __name__ == "__main__":
